package view
{
	import adobe.utils.CustomActions;
	import com.greensock.TweenLite;
	import events.CardEvent;
	import flash.events.MouseEvent;
	import flash.geom.Point;
	import me.rainssong.display.DisplayCore;
	import me.rainssong.math.MathCore;
	import me.rainui.RainUI;
	import me.rainui.components.Container;
	import me.rainui.events.RainUIEvent;
	import view.card.Card;
	import view.card.characterCard.Friend;
	
	/**
	 * @date 2016/12/10 15:10
	 * @author rainssong
	 * @blog http://blog.sina.com.cn/rainssong
	 * @homepage http://www.rainssong.com
	 * @weibo http://www.weibo.com/rainssong
	 */
	public class CardGroup extends Container
	{
		private var cards:Vector.<Card>
		public var minGap:Number = 40;
		public var maxGap:Number = 80;
		public var owner:Player;
		
		public function CardGroup(dataSource:Object = null)
		{
			super(dataSource);
		}
		
		override protected function initialize():void
		{
			cards = new Vector.<Card>();
			this._height = Card.CardHeight;
			super.initialize();
			
			this.addEventListener(MouseEvent.CLICK, onClickCard);
			this.addEventListener(MouseEvent.MOUSE_OVER, onMouseOver);
			this.addEventListener(MouseEvent.MOUSE_OUT, onMouseOut);
		}
		
		private function onMouseOut(e:MouseEvent):void
		{
			for (var i:int = 0; i < cards.length; i++)
			{
				cards[i] .skin.y = 0;
				setChildIndex(cards[i],i);
			}
			
			
			//if (e.target is Card)
			//{
				//var i:int = cards.indexOf(e.target as Card);
				//cards[i].y = 0;
				//setChildIndex(cards[i], i);
			//}
			
			
			
		}
		
		private function onMouseOver(e:MouseEvent):void
		{
			var card:Card
			if (e.target is Card)
			{
				card = e.target as Card;
				card.skin.y = -20;
				setChildIndex(card, cards.length);
				
			}
		}
		
		public function addCard(card:Card):void
		{
			cards.push(card);
			if (card.group)
				card.group.draw(card);
			card.owner = owner;
			card.group = this;
			
			DisplayCore.moveWithPosition(card, this)
			
			if (parent)
				this.parent.addChild(this);
			
			resort();
		}
		
		private function onClickCard(e:MouseEvent):void
		{
			if (e.target is Card)
				dispatchEvent(new CardEvent(CardEvent.PLAY_CARD, e.target as Card));
		}
		
		public function draw(card:Card):void
		{
			var i:int = cards.indexOf(card);
			if (i >= 0)
			{
				cards.splice(i, 1);
				resort();
			}
		
		}
		
		public function removeCard(card:Card):void
		{
			var i:int = cards.indexOf(card)
			removeCardIndex(i);
		}
		
		public function removeAllCards():void 
		{
			while (cards.length)
			{
				removeCardIndex(0);
			}
		}
		
		public function pop():Card
		{
			var card:Card = lastCard;
			draw(card);
			return card;
		}
		
		public function shift():Card
		{
			var card:Card = cards[0];
			draw(card);
			return card;
		}
		
		public function removeCardIndex(i:int):void
		{
			if (i >= 0)
			{
				var card:Card = cards[i];
				cards.splice(i, 1);
				
				card.remove();
				resort();
			}
		}
		
		public function resort():void
		{
			if (cards.length == 0)
				return;
			
			var gap:Number = MathCore.getRangedNumber(600 / cards.length, minGap, maxGap);
			
			var offset:Number = RainUI.stage.stageWidth * 0.5 - (gap * (cards.length - 1) + Card.CardWidth) * 0.5;
			
			for (var i:int = 0; i < cards.length; i++)
			{
				TweenLite.killTweensOf(cards[i], true);
				TweenLite.to(cards[i], 0.5, {x: i * gap + offset, y: 0});
			}
		}
		
		override public function resize():void
		{
			super.resize();
			resort();
		}
		
		/* DELEGATE Vector.<T> */
		
		public function get length():int
		{
			return cards.length;
		}
		
		public function get lastCard():Card
		{
			if (cards.length <= 0)
				return null;
			
			return cards[cards.length - 1];
		}
		
		public function get lastButOneCard():Card
		{
			if (cards.length <= 1)
				return null;
			
			return cards[cards.length - 2];
		}
		
		public function update():void
		{
			for (var i:int = 0; i < cards.length; i++) 
			{
				cards[i].update();
			}
		}
		
		public function newRound():void 
		{
			//for (var i:int = 0; i < cards.length; i++) 
			//{
				//cards[i].newRound();
			//}
			update();
		}
		
		public function getCard(i:int):Card 
		{
			return cards[i]
		}
		
		public function getCardByClass(cls:Class):Card 
		{
			for (var i:int = 0; i < cards.length; i++) 
			{
				if (cards[i] as cls)
					return cards[i];
			}
			return null;
		}
		

		
		

	
	}

}